@@ -10,6 +10,7 @@ To be used by creating a `fabfile.py` with the following two line::
   env.django_databases.append(...)
   env.django_admin_tests.append(...)
   env.simple_doctests.append(...)
+  env.tolerate_sphinx_warnings = True
   
 Where "foobar" is the name of your main package.
   
@@ -17,11 +18,19 @@ This fablib uses the following `env` keys:
 
 - `django_databases` : a list of directories where a manage.py exists
   and for which initdb_demo should be executed.
+- `tolerate_sphinx_warnings` : whether `sphinx-build html` should tolerate warnings.
+
+- env.editor : your preferred text editor
+
 - (consult the source code)
 
 """
 import os
 import sys
+import doctest
+import pkg_resources
+import textwrap
+
 #~ def clean_sys_path():
     #~ # print sys.path
     #~ if sys.path[0] == '':
@@ -48,11 +57,31 @@ from timtools.tools.synchronizer import Synchronizer
 from fabric.api import env, local, task, prompt
 from fabric.utils import abort, fastprint, puts, warn
 from fabric.contrib.console import confirm
-
+from fabric.api import lcd
 
 #~ LONG_DATE_FORMAT = 
 
 
+PROJECTS = []
+
+
+
+class RstFile(object):
+    def __init__(self,local_root,url_root,parts):
+        self.path = local_root.child(*parts)
+        self.url = url_root + "/".join(parts)
+        #~ self.parts = parts
+        
+      
+
+class Project(object):
+    def __init__(self,i,name):
+        self.index = i
+        self.name = name
+        self.dist = pkg_resources.get_distribution(name)
+        self.module = __import__(name)
+    
+
 def setup_from_project(main_package):
   
     #~ env.docs_rsync_dest = 'luc@lino-framework.org'
@@ -61,6 +90,10 @@ def setup_from_project(main_package):
     #~ HOME = Path(os.path.expanduser("~"))
     #~ REMOTE = AttrDict(
       #~ lf='lino-framework.org')
+      
+    for i,prj in enumerate(env.projects.split()):
+        PROJECTS.append(Project(i,prj))
+      
     env.ROOTDIR = Path().absolute()
 
     env.project_name = env.ROOTDIR.absolute().name
@@ -121,6 +154,26 @@ def rmtree_after_confirm(p):
     p.rmtree()
     
 
+@task(alias='summary')
+def summary(*cmdline_args):
+    headers = (
+      #~ '#','Location',
+      'Project','Old version','New version')
+
+    def cells(self):
+        desc = "`%s <%s>`__ -- %s" % (
+            self.name,
+            self.module.SETUP_INFO['url'],
+            self.module.SETUP_INFO['description'],
+            )        
+        return (
+            '\n'.join(textwrap.wrap(desc,40)),
+            self.dist.version,
+            self.module.__version__)
+        
+    print rstgen.table(headers,[cells(p) for p in PROJECTS])
+        
+  
 @task(alias='api')
 def build_api(*cmdline_args):
     """
@@ -148,9 +201,10 @@ def build_api(*cmdline_args):
 @task(alias='html')
 def build_html(): #~ def build_html(*cmdline_args):
     """
-    Build sphinx docs.
+    write_readme + build sphinx html docs.
     """
     write_readme()
+    write_release_notes()
     #~ print cmdline_args
     args = ['sphinx-build','-b','html']
     #~ args += cmdline_args
@@ -193,6 +247,26 @@ def clean_html(*cmdline_args):
     """
     rmtree_after_confirm(env.BUILDDIR)
     
+#~ @task(alias='pub')
+#~ def publish_all():
+    #~ """
+    #~ Run `publish_docs` followed by `hg_push`.
+    #~ """
+    #~ publish_docs()
+    #~ hg_push()
+    
+    
+@task(alias='prep')
+def prepare():
+    """
+    Sames as `fab test html`.
+    """
+    run_tests()
+    build_html()
+    
+    
+    
+  
 @task(alias='pub')
 def publish_docs():
     """
@@ -200,36 +274,39 @@ def publish_docs():
     """
     #~ from fabric.context_managers import cd
     cwd = Path(os.getcwd())
-    env.BUILDDIR.chdir()
-    #~ with cd(env.BUILDDIR):
-    #~ addr = env.user+'@'+REMOTE.lf
-    args = ['rsync','-r']
-    args += ['--verbose'] 
-    args += ['--progress'] # show progress
-    args += ['--delete'] # delete files in dest
-    args += ['--times'] # preserve timestamps
-    args += ['--exclude','.doctrees'] 
-    args += ['./'] # source
-    args += [env.docs_rsync_dest+':~/public_html/'+env.project_name] # dest
-    cmd = ' '.join(args)
-    puts("%s> %s" % (os.getcwd(), cmd))
-    #~ confirm("yes")
-    local(cmd)
-    cwd.chdir()
+    with lcd(env.BUILDDIR):
+        #~ env.BUILDDIR.chdir()
+        #~ with cd(env.BUILDDIR):
+        #~ addr = env.user+'@'+REMOTE.lf
+        args = ['rsync','-r']
+        args += ['--verbose'] 
+        args += ['--progress'] # show progress
+        args += ['--delete'] # delete files in dest
+        args += ['--times'] # preserve timestamps
+        args += ['--exclude','.doctrees'] 
+        args += ['./'] # source
+        args += [env.docs_rsync_dest+':~/public_html/'+env.project_name] # dest
+        cmd = ' '.join(args)
+        puts("%s> %s" % (os.getcwd(), cmd))
+        #~ confirm("yes")
+        local(cmd)
+        #~ cwd.chdir()
     #~ return subprocess.call(args)
 
 def run_in_django_databases(admin_cmd,*more):
     for db in env.django_databases:
-        p = env.ROOTDIR.child(db)
-        # cmd = 'python manage.py initdb --noinput'
-        args = ["django-admin"] 
-        args += [admin_cmd]
-        args += more
-        #~ args += ["--noinput"]
-        args += ["--settings=settings"]
-        args += [" --pythonpath=%s" % p.absolute()]
-        cmd = " ".join(args)
-        local(cmd)
+        p = env.ROOTDIR.child(*db.split('/'))
+        with lcd(p):
+            # cmd = 'python manage.py initdb --noinput'
+            #~ args = ["django-admin"] 
+            args = ["python manage.py"] 
+            args += [admin_cmd]
+            args += more
+            #~ args += ["--noinput"]
+            #~ args += ["--settings=settings"]
+            args += [" --pythonpath=%s" % p.absolute()]
+            cmd = " ".join(args)
+            local(cmd)
   
 @task(alias="initdb")
 def initdb_demo():
@@ -257,9 +334,9 @@ def initdb_demo():
     
     run_in_django_databases('initdb_demo',"--noinput")
 
-@task()
-def runserver():
-    run_in_django_databases('runserver')
+#~ @task()
+#~ def runserver():
+    #~ run_in_django_databases('runserver')
     
         
 @task()
@@ -305,8 +382,9 @@ Sphinx doctest failed with exit code %s
 @task(alias='sdist')
 def setup_sdist():
     """
-    Write source districution archive file.
+    Write source distribution archive file.
     """
+    pipy_register()
     #~ puts(env.sdist_dir)
     args = ["python", "setup.py"]
     args += [ "sdist", "--formats=gztar" ]
@@ -315,7 +393,11 @@ def setup_sdist():
     #~ run_setup('setup.py',args)
   
 @task(alias='upload')
-def setup_sdist_upload():
+def pypi_upload():
+    """
+    Upload sourcxe distribution to PyPI.
+    """
+    pipy_register()
     args = ["python", "setup.py"]
     args += ["sdist", "--formats=gztar" ]
     args += ["--dist-dir", env.sdist_dir.child(env.SETUP_INFO['name'])]
@@ -336,20 +418,19 @@ def setup_sdist_upload():
     
   
 @task(alias='reg')
-def setup_register():
+def pipy_register():
+    """
+    Register to PyPI.
+    """
     args = ["python", "setup.py"]
     args += ["register"]
     #~ run_setup('setup.py',args)
     local(' '.join(args))
 
-class RstFile(object):
-    def __init__(self,local_root,url_root,parts):
-        self.path = local_root.child(*parts)
-        self.url = url_root + "/".join(parts)
-        #~ self.parts = parts
-        
-      
 def get_blog_entry(today):
+    """
+    Return an RstFile object representing the blog entry for that date.
+    """
     local_root = env.work_root.child(env.blogger_project)
     parts = ('docs','blog',str(today.year),today.strftime("%m%d.rst"))
     #~ return blogdir.child(*parts)
@@ -369,7 +450,8 @@ def edit_blog_entry():
         if confirm("Create file %s?" % entry.path):
             txt = rstgen.header(1,today.strftime(env.long_date_format))
             entry.path.write_file(txt)
-    args = [env.editor]
+            entry.path.parent.child('index.rst').set_times() # touch it for Sphinx
+    args = [ env.editor ]
     args += [entry.path]
     local(' '.join(args))
   
@@ -378,7 +460,6 @@ def checkin():
     """
     Checkin & push to repository, using today's blog entry as commit message.
     """
-    write_readme()
     entry = get_blog_entry(datetime.date.today())
     #~ entry = Path(env.ROOTDIR,'..',env.blogger_project,*parts)
     #~ print env.ROOTDIR.parent.absolute()
@@ -395,6 +476,52 @@ def checkin():
     local(cmd)
     local("hg push %s" % env.project_name)
     
+@task()
+def write_release_notes():
+    """
+    Generate docs/releases/x.y.z.rst file from setup_info.
+    """
+    notes = Path(env.ROOTDIR,'docs','releases','%s.rst' % env.SETUP_INFO['version'])
+    if notes.exists():
+        return
+    must_confirm("Create %s" % notes.absolute())
+    context = dict(date=datetime.date.today().strftime(env.long_date_format))
+    context.update(env.SETUP_INFO)
+    txt = """\
+==========================
+Version %(version)s
+==========================
+
+%(date)s
+
+I am pleased to announce the release of 
+version %(version)s of `%(name)s <%(url)s>`__.
+
+%(author)s
+
+List of changes
+===============
+
+New features
+------------
+
+Optimizations
+-------------
+
+Bugfixes
+--------
+
+Manual tasks after upgrade
+--------------------------
+
+
+""" % context
+    notes.write_file(txt)
+    notes.parent.child('index.rst').set_times()
+    args = [ env.editor ]
+    args += [notes.absolute()]
+    local(' '.join(args))
+    
 @task()
 def write_readme():
     """
@@ -417,31 +544,26 @@ Read more on %(url)s
 """ % env.SETUP_INFO
     if readme.read_file() == txt:
         return 
-    if not confirm("Overwrite %s" % readme.absolute()):
-        abort
+    must_confirm("Overwrite %s" % readme.absolute())
     readme.write_file(txt)
-    cmd = "touch " + env.DOCSDIR.child('index.rst')
-    local(cmd)
-    setup_register()
-
+    env.DOCSDIR.child('index.rst').set_times()
+    #~ cmd = "touch " + env.DOCSDIR.child('index.rst')
+    #~ local(cmd)
+    #~ pipy_register()
+    
 @task(alias='t2')
 def run_django_admin_tests():
+    """
+    Run `django-admin test` for each `env.django_admin_tests`.
+    """
     for prj in env.django_admin_tests:
-        cmd = "django-admin test --settings=%s --verbosity=0 --traceback" % prj
-        local(cmd)
-  
-@task(alias='t7')
-def run_bash_tests():
-    """
-    Run the commands in `env.bash_tests`.
-    """
-    for cmd in env.bash_tests:
+        cmd = "django-admin test --settings=%s --verbosity=0 --failfast --traceback" % prj
         local(cmd)
   
 @task(alias='t3')
 def run_django_doctests():
     """
-    run `django-admin test` in the `docs` dir for each `django_doctests`
+    Run `django-admin test` in the `docs` dir for each `env.django_doctests`
     """
     #~ must_exist(env.DOCSDIR.child('manage.py'))
     #~ env.DOCSDIR.chdir()
@@ -458,25 +580,37 @@ def run_django_doctests():
 @task(alias='t4')
 def run_simple_doctests():
     """
-    Run a normal doctest for files specified in `simple_doctests`.
+    Run a simple doctest for files specified in `env.simple_doctests`.
     """
     os.environ['DJANGO_SETTINGS_MODULE']='lino.projects.std.settings'
-    for prj in env.simple_doctests:
-        cmd = "python -m doctest %s" % prj
+    for filename in env.simple_doctests:
+        cmd = "python -m doctest %s" % filename
         #~ print cmd
         local(cmd)
+        #~ doctest.testfile(filename, module_relative=False,encoding='utf-8')
 
 @task(alias='t5')
 def run_django_databases_tests():    
+    """
+    Run "manage.py test" for each `env.django_databases`.
+    """
     run_in_django_databases('test',"--noinput")
 
 @task(alias='t6')
 def run_setup_tests():    
     """
-    Runs certain tests related to packaging.
+    Run hardcoded tests related to packaging.
     """
-    unittest.main(argv=['fab','SetupTest'],module=__name__)
+    unittest.main(argv=['fab','SetupTest'],module=__name__,exit=False)
 
+@task(alias='t7')
+def run_bash_tests():
+    """
+    Run the commands in `env.bash_tests`.
+    """
+    for cmd in env.bash_tests:
+        local(cmd)
+  
 @task(alias='test')
 def run_tests():
     """
@@ -489,6 +623,7 @@ def run_tests():
     run_django_databases_tests() # t5
     run_setup_tests() # t6
     run_bash_tests() # t7
+    print "run_tests() done"
     
 
 #~ @task(alias='listpkg')
