@@ -37,11 +37,14 @@ from unipath import Path
 
 #~ from django.core.exceptions import ObjectDoesNotExist, MultipleObjectsReturned
 
-from docutils import nodes
+from docutils import nodes, utils
 from docutils.parsers.rst import directives
+from docutils.parsers.rst import roles
 from sphinx.util.compat import Directive
+from sphinx.util.nodes import split_explicit_title
 
 from djangosite.utils import rstgen
+from djangosite.utils import i2d
 
 #~ class ScreenshotDirective(directives.images.Image):
     #~ """
@@ -190,17 +193,19 @@ class InsertInputDirective(Directive):
     to be forwarded to `state_machine.insert_input()`.
     """
     has_content = True
+    debug = False
     def get_rst(self):
         raise NotImplementedErrro()
         
     def run(self):
         out = self.get_rst()
         env = self.state.document.settings.env
-        #~ print env.docname
-        #~ print '-' * 50
-        #~ print out
-        #~ print '-' * 50
-        #~ sys.exit()
+        if self.debug:
+            print env.docname
+            print '-' * 50
+            print out
+            print '-' * 50
+            #~ sys.exit()
         self.state_machine.insert_input(out.splitlines(),out)
         return []
 
@@ -208,17 +213,43 @@ class InsertInputDirective(Directive):
 
 class Py2rstDirective(InsertInputDirective):
     """
-    This works, but is not used.
+    Run a Python code block and interpret the output as it it 
+    were rst source.
     """
     def get_rst(self):
         code = '\n'.join(self.content)
         old = sys.stdout
         buffer = StringIO()
         sys.stdout = buffer
-        exec(code,{})
+        context = dict()
+        from django.conf import settings
+        #~ context = dict(settings=settings)
+        context.update(settings.SITE.modules)
+        context.update(self=self)
+        context.update(env=self.state.document.settings.env)
+        exec(code,context)
         sys.stdout = old
         return buffer.getvalue()
         
+#~ class DjangoTableDirective(InsertInputDirective):
+    #~ def get_rst(self):
+        #~ assert len(self.content) == 1
+        #~ code = '\n'.join(self.content)
+        
+        #~ from django.conf import settings    
+        #~ print .jobs.Candidatures.request(limit=5).to_rst()
+        
+        #~ code = """
+        #~ """
+        #~ old = sys.stdout
+        #~ buffer = StringIO()
+        #~ sys.stdout = buffer
+        #~ context = dict()
+        #~ context.update(settings.SITE.modules)
+        #~ context = dict(settings=settings)
+        #~ exec(code,context)
+        #~ sys.stdout = old
+        #~ return buffer.getvalue()
         
 class TextImageDirective(InsertInputDirective):
     """
@@ -421,7 +452,7 @@ class YearBlogIndexDirective(InsertInputDirective):
     
       
     def get_rst(self):
-        from north import babel
+        from north import dbutils
   
         #~ year = self.arguments[0]
         env = self.state.document.settings.env
@@ -442,7 +473,7 @@ class YearBlogIndexDirective(InsertInputDirective):
             
             text += """        
             
-.. |M%02d| replace::  **%s**""" % (month,babel.monthname(month))
+.. |M%02d| replace::  **%s**""" % (month,dbutils.monthname(month))
             
             weeknum = None
             #~ text += "\n  |br| Mo Tu We Th Fr Sa Su "
@@ -520,8 +551,57 @@ def docname_to_day(year,s):
             #~ entries = env.changed_items.setdefault(item,dict())
             #~ entries.setdefault(env.docname)
             
+            
+
+def get_blog_url(today):
+    blogger_project = "lino"
+    url_root = "http://code.google.com/p/%s/source/browse/" % blogger_project
+    parts = ('docs','blog',str(today.year),today.strftime("%m%d.rst"))
+    url = url_root + "/".join(parts)
+    return url
+
+
+def blogref_role(name, rawtext, text, lineno, inliner,options={}, content=[]):
+    """
+    Inserts a reference to the blog entry of the specified date.
+    
+    Instead of writing ``:doc:`/blog/2011/0406```
+    it is better to write ``:blogref:`20110406```
+    because the latter works between Sphinx trees and also supports archived blog entries.
+    
+    """
+    # thanks to http://docutils.sourceforge.net/docs/howto/rst-roles.html
+    # this code originally from roles.pep_reference_role
+    #~ print 20130315, rawtext, text, utils.unescape(text)
+    has_explicit_title, title, target = split_explicit_title(text)
+    try:
+        date = i2d(int(target))
+    except ValueError:
+        msg = inliner.reporter.error(
+            'Invalid text %r: must be an integer date of style "20130315" .'
+            % text, line=lineno)
+        prb = inliner.problematic(rawtext, rawtext, msg)
+        return [prb], [msg]
+    #~ print repr(env)
+    #~ raise Exception(20130315)
+    #~ ref = inliner.document.settings.pep_base_url
+           #~ + inliner.document.settings.pep_file_url_template % date)
+    roles.set_classes(options)
+    #~ from django.conf import settings
+    #~ shown_text = settings.SITE.dtos(date)
+    env = inliner.document.settings.env        
+    if not has_explicit_title:
+        title = date.strftime(env.settings.get('today_fmt','%Y-%m-%d'))
+    title = utils.unescape(title)
+    return [nodes.reference(rawtext, title, 
+                            refuri=get_blog_url(date),
+                            **options)], []
+    
+    
+            
 def configure(filename,globals_dict):
     """
+    To be callsed from inside the Sphinx `conf.py`.
     This contains the things that all my Sphinx docs configuration 
     files have in common.
     
@@ -558,7 +638,8 @@ def configure(filename,globals_dict):
     would occur when this happens later while importing one of the models modules.
     """
     from django.conf import settings
-    settings.SITE.startup()
+    settings.SITE # must at least access some variable in the settings
+    #~ settings.SITE.startup()
     globals_dict.update(setup=setup)
 
         
@@ -600,10 +681,28 @@ def setup(app):
     app.add_directive('blogger_year', YearBlogIndexDirective)
     app.add_directive('blogger_index', MainBlogIndexDirective)
     app.add_directive('textimage', TextImageDirective)
+    roles.register_canonical_role('blogref', blogref_role)
+    
     setup2(app)
     #~ app.add_directive('screenshot', ScreenshotDirective)
     #~ app.add_config_value('screenshots_root', '/screenshots/', 'html')
 
     #~ from djangosite.utils import doctest
     #~ doctest.setup(app)
-    
\ No newline at end of file
+    
+#~ print "OK"    
+
+
+def version2rst(self,m):
+    """
+    used in docs/released/index.rst
+    """
+    v = m.__version__
+    if v.endswith('+'):
+        v = v[:-1]
+        print "The current stable release is :doc:`%s`." % v 
+    elif v.endswith('pre'):
+        print "We're currently working on  :doc:`%s`." % v[:-3]
+    else:
+        print "The current stable release is :doc:`%s`." % v 
+        #~ print "We're currently working on :doc:`coming`."
